#ifndef _DISCOVER_H_
#define _DISCOVER_H_

#if defined (CAMEO_DISCOVERY) || defined (CAMEO_DISCOVER_SEND_IGMP_REPORT)

extern void DiscoverHandler (uchar * pkt, unsigned dest, unsigned src, unsigned len);

#define DISCOVERY_SUPPORT_MULICAST_PKT  1

#if DISCOVERY_SUPPORT_MULICAST_PKT
#define DISCOVER_IPADDR                 0xEFFFFF64L
#else
#define DISCOVER_IPADDR                 0xFFFFFFFFL
#endif

#define SYS_DISCOV_SPECIFIC_IP_GROUP    0xEFFFFF64L
#define SYS_DISCOV_SPECIFIC_MAC_GROUP   0x01, 0x00, 0x5e, 0x7f, 0xff, 0x64

#define UDP_DISCOV_BROADCASTMODE_PORT   0xFC01 /* udp src port from smart console utility on brocast mode */
#define UDP_DISCOV_MONITOR_PORT         0xFC02 /* udp src port from smart console utility on monitor mode */
#define UDP_DISCOV_AGENT_PORT           0xFC03 /* device agent udp src port */
#define UDP_TEST_MODE_PORT              0xFC04
#define UDP_AGENT_HELLO_PORT            0xFC06
#define UDP_DISCOV_MULTICASTMODE_PORT   0xFC07 /* udp src port from smart console utility on multicast mode */

#ifndef IP_ADDR_LEN
#define IP_ADDR_LEN 4
#endif

#if DISCOVERY_SUPPORT_MULICAST_PKT
#define ip_addrisdiscovery(a) ((a[0] == 0xEF) && (a[1] == 0xFF) && (a[2] == 0xFF) && (a[3] == 0x64))
#else /* BROADCAST_PKT */
#define ip_addrisdiscovery(a) ((a[0] == 0xFF) && (a[1] == 0xFF) && (a[2] == 0xFF) && (a[3] == 0xFF))
#endif

/* netgear signature */
#define TRENDNET_SIGNATURE              0x01945AD8L
#define CAMEO_SIGNATURE                 26499800L
#define DLINK_SIGNATURE                 26499808L
#define NETGEAR_SIGNATURE               0x0C07D2F2L

/* protocol version*/
#define PROTOCOL_VERSION                2

/* interface version */
#define INTERFACE_VERSION               6

/* family type */
#define FAMILY_TYPE_GENERAL             0
#define FAMILY_TYPE_WEB_SWITCH          1
#define FAMILY_TYPE_AP                  2

/* product type */
#define PRODUCT_TYPE_WEB_SWITCH         1

/* discovery header, total length is 40 bytes */
#define DISCOV_RESERVED_LEN             10
#define MAC_ADDR_LEN                    6

typedef struct discov_hdr_s
{
	unsigned short  proto_ver;
	unsigned short  family_type;
	unsigned char   agent_mac[MAC_ADDR_LEN];
	unsigned short  inter_ver;
	unsigned short  command;
	unsigned short  status;
	unsigned int    signature;
	unsigned char   manager_mac[MAC_ADDR_LEN];
	unsigned short  seq_id;
	unsigned char   in_port;
	unsigned char   out_port;
	unsigned char   reserved[DISCOV_RESERVED_LEN];
} discov_hdr_t;

typedef struct igmp_hdr_s {
	unsigned char   version_type;
	unsigned char   unused;
	unsigned short  igmpchksum;
	unsigned int    groupAddress;
} igmp_hdr_t;

#endif
#endif /* _DISCOVER_H_ */
